"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getUsedComponents = exports.getMpEventHanlderName = exports.generateWxml = void 0;
const chalk_1 = __importDefault(require("chalk"));
const weapps_core_1 = require("../../weapps-core");
const xml_js_1 = require("xml-js");
const mp_1 = require("../config/mp");
const materials_1 = require("./materials");
const weapp_1 = require("../util/weapp");
const name_mangler_1 = __importDefault(require("../util/name-mangler"));
const error = chalk_1.default.redBright;
function generateWxml(widgets, docTag, wxmlDataPrefix, ctx, usingComponents, nodeTransform) {
    var _a;
    const nameMangler = ctx.isProduction
        ? new name_mangler_1.default({ blackList: mp_1.builtinMpTags })
        : undefined;
    const xmlJson = {
        elements: [
            {
                type: 'element',
                name: 'wxs',
                attributes: {
                    src: '/common/utils.wxs',
                    module: 'wxsUtils',
                },
                elements: [],
                _order: -1,
                _parent: null,
            },
        ].concat(createXml(widgets)),
    };
    if (ctx.isPage) {
        const originElements = xmlJson.elements;
        if (originElements === null || originElements === void 0 ? void 0 : originElements.length) {
            if (!((_a = originElements[0]) === null || _a === void 0 ? void 0 : _a.attributes)) {
                originElements[0].attributes = {};
            }
            originElements[0].attributes['data-weui-theme'] = 'light';
        }
        xmlJson.elements = [
            {
                type: 'element',
                name: 'block',
                attributes: {
                    ['wx:if']: '{{weDaHasLogin}}',
                },
                elements: originElements,
                _order: -1,
                _parent: null,
            },
        ];
    }
    function createXml(widgets, parent = null, parentForNodes = []) {
        var _a;
        const elements = [];
        for (const id in widgets) {
            const { xComponent, xProps, properties, xIndex } = widgets[id];
            const { data: data0 = {}, listeners = [], directives = {}, staticResourceAttribute = ['src'], } = xProps || {};
            const data = Object.assign({}, data0);
            if (directives.waIf && directives.waIf.value === false) {
                continue;
            }
            if (!xComponent) {
                const slotNodes = createXml(properties, parent, parentForNodes);
                slotNodes.forEach((node) => {
                    var _a;
                    node.attributes.slot = id;
                    (_a = parent === null || parent === void 0 ? void 0 : parent.elements) === null || _a === void 0 ? void 0 : _a.push(node);
                });
                if (parent) {
                    delete parent.attributes[id];
                }
                continue;
            }
            const componentKey = xComponent.moduleName + ':' + xComponent.name;
            const helpMsg = `Please check component(${id}) in component tree of ${docTag}.`;
            const materialLib = ctx.materialLibs.find((lib) => lib.name === xComponent.moduleName);
            const miniprogramPlugin = (_a = ctx.miniprogramPlugins) === null || _a === void 0 ? void 0 : _a.find((plugin) => plugin.name === xComponent.moduleName);
            if (!materialLib && !miniprogramPlugin) {
                console.error(error(`Component lib(${xComponent.moduleName}) not found. ${helpMsg}`));
                continue;
            }
            const componentProto = materialLib
                ? materialLib.components.find((comp) => comp.name === xComponent.name)
                : miniprogramPlugin === null || miniprogramPlugin === void 0 ? void 0 : miniprogramPlugin.componentConfigs.find((comp) => comp.name === xComponent.name);
            if (!componentProto) {
                console.error(error(`Component(${xComponent.name}) not found in lib(${xComponent.moduleName}). ${helpMsg}`));
                continue;
            }
            const { tagName, path } = materials_1.getWxmlTag(xComponent, ctx, nameMangler);
            if (path) {
                usingComponents[tagName] = path;
            }
            if (tagName === 'slot') {
                elements.push({
                    type: 'element',
                    name: tagName,
                    attributes: { name: data0.name.value },
                    elements: [],
                    _order: xIndex || 0,
                    _parent: null,
                });
                continue;
            }
            let curForNodes = parentForNodes;
            if (directives.waFor && directives.waFor.value) {
                curForNodes = [...curForNodes, id];
            }
            const attrPrefix = `${wxmlDataPrefix.widgetProp}${id}${curForNodes
                .map((forNodeId) => `[${wxmlDataPrefix.forIndex}${forNodeId}]`)
                .join('')}.`;
            const idAttr = curForNodes.length < 1
                ? id
                : `{{'${id}'${curForNodes
                    .map((forNodeId) => `+ '-' + ${wxmlDataPrefix.forIndex}${forNodeId}`)
                    .join('')}}}`;
            const node = {
                type: 'element',
                name: tagName,
                attributes: {
                    id: idAttr,
                    style: `{{${attrPrefix}style}}`,
                    [mp_1.getClassAttrName(tagName)]: `{{${attrPrefix}className}}`,
                },
                elements: [],
                _order: xIndex || 0,
                _parent: parent,
            };
            const { mustEmptyStyle } = componentProto.meta || {};
            if (mustEmptyStyle) {
                delete node.attributes.style;
            }
            if (directives.waIf && directives.waIf.value) {
                node.attributes['wx:if'] = getAttrBind(directives.waIf, `${attrPrefix}_waIf`);
            }
            if (directives.waFor && directives.waFor.value) {
                node.attributes['wx:for'] = getAttrBind(directives.waFor, `${wxmlDataPrefix.widgetProp}${id}${parentForNodes
                    .map((forNodeId) => `[${wxmlDataPrefix.forIndex}${forNodeId}]`)
                    .join('')}`);
                node.attributes['wx:for-index'] = wxmlDataPrefix.forIndex + id;
                node.attributes['wx:key'] = 'id';
            }
            const compSchema = componentProto.dataForm;
            for (const prop in data) {
                if (compSchema) {
                    const fieldDef = compSchema[prop];
                    if (!fieldDef) {
                        console.log(error(`Prop(${prop}) does not exist on ${componentKey}. ${helpMsg}`));
                        continue;
                    }
                    if (fieldDef.readOnly) {
                        if (fieldDef.hasOwnProperty('default')) {
                            node.attributes[prop] = fieldDef.default;
                        }
                        else {
                            console.error(error(`Readonly property(${prop}) of ${componentKey} must have a default value. ${helpMsg}`));
                        }
                        continue;
                    }
                }
                xmlJsonSetCustomAttr(node, prop, getAttrBind(data[prop], `${attrPrefix}${prop}`, staticResourceAttribute.includes(prop)), xComponent);
            }
            const { inputProps, syncProps } = componentProto.meta || {};
            const syncConfigs = syncProps || inputProps || {};
            Object.entries(syncConfigs).map(([prop, config]) => {
                const configs = Array.isArray(config) ? config : [config];
                configs.forEach(({ changeEvent: evtName }) => {
                    const sep = getEventBindSep(tagName);
                    node.attributes[`bind${sep}${evtName}`] = getMpEventHanlderName(id, evtName);
                });
            });
            listeners.forEach((l) => {
                const evtName = getMpEventName(l.trigger);
                const modifiers = l;
                node.attributes[getMpEventAttr(evtName, modifiers, tagName)] = getMpEventHanlderName(id, evtName, modifiers);
            });
            const compConfig = componentProto.compConfig;
            if (compConfig && compConfig.pluginConfig) {
                if (compConfig.pluginConfig.attributes) {
                    Object.assign(node.attributes, compConfig.pluginConfig.attributes);
                }
                if (compConfig.pluginConfig.componentPath) {
                    usingComponents[tagName] = compConfig.pluginConfig.componentPath;
                }
            }
            node.elements = node.elements.concat(createXml(properties, node, curForNodes));
            if (tagName === 'swiper' ||
                componentKey === 'weda:Swiper' ||
                componentKey === 'gsd-h5-react:Swiper') {
                node.elements = node.elements.map((item, index) => {
                    var _a, _b;
                    let _c = item.attributes || {}, { ['wx:for']: wxFor, ['wx:for-index']: wxForIndex, ['wx:key']: wxKey } = _c, itemRestKey = __rest(_c, ['wx:for', 'wx:for-index', 'wx:key']);
                    if (item.name !== 'swiper-item') {
                        let SwiperItem = {
                            type: 'element',
                            name: 'swiper-item',
                            attributes: {
                                id: `${((_a = item.attributes) === null || _a === void 0 ? void 0 : _a.id) || ((_b = node === null || node === void 0 ? void 0 : node.attributes) === null || _b === void 0 ? void 0 : _b.id) + index}-item`,
                                'wx:for': wxFor,
                                'wx:for-index': wxForIndex,
                                'wx:key': wxKey,
                            },
                            elements: [],
                            _order: index || 0,
                            _parent: node,
                        };
                        SwiperItem.elements = [
                            Object.assign(Object.assign({}, item), { attributes: Object.assign({}, itemRestKey), _parent: SwiperItem }),
                        ];
                        return SwiperItem;
                    }
                    else {
                        return item;
                    }
                });
            }
            nodeTransform && nodeTransform(widgets[id], node);
            elements.push(node);
        }
        return elements.sort((a, b) => a._order - b._order);
    }
    return xml_js_1.js2xml(xmlJson, {
        spaces: '\t',
    });
}
exports.generateWxml = generateWxml;
function xmlJsonSetCustomAttr(node, prop, value, comp) {
    if (mp_1.builtinWigetProps.indexOf(prop) > -1) {
        console.error(error('Builtin prop(' + prop + ') is not allowed for custom component'), comp);
        return;
    }
    node.attributes[prop] = value;
    if (prop === mp_1.textContentPropName) {
        node.elements.push({ type: 'text', text: value });
    }
}
const evtNameMap = {
    __weapps_action_trigger_click: 'tap',
};
function getMpEventName(originalName) {
    return evtNameMap[originalName] || originalName.replace(/\./g, '_');
}
function getMpEventHanlderName(widgetId, evtName, modifier = {}) {
    if (mp_1.builtinMpEvents.indexOf(evtName) === -1) {
        modifier = {};
    }
    return `on${widgetId}$${getMpEventName(evtName)}${modifier.isCapturePhase ? '$cap' : ''}${modifier.noPropagation ? '$cat' : ''}`;
}
exports.getMpEventHanlderName = getMpEventHanlderName;
function getMpEventAttr(evtName, modifier, tagName) {
    if (mp_1.builtinMpEvents.indexOf(evtName) === -1) {
        modifier = {};
    }
    let prefix = modifier.isCapturePhase ? 'capture-' : '';
    prefix += modifier.noPropagation ? 'catch' : 'bind';
    const sep = getEventBindSep(tagName);
    return prefix + sep + evtName;
}
function getEventBindSep(tagName) {
    return mp_1.nativeCompWhiteList.includes(tagName.toLowerCase()) ? '' : ':';
}
function getUsedComponents(widgets, usedCmps = {}) {
    weapp_1.walkThroughWidgets(widgets, (id, widget) => {
        const { xComponent } = widget;
        if (!xComponent)
            return;
        const { moduleName, name } = xComponent;
        if (!usedCmps[moduleName]) {
            usedCmps[moduleName] = new Set();
        }
        usedCmps[moduleName].add(name);
    });
    return usedCmps;
}
exports.getUsedComponents = getUsedComponents;
function getAttrBind(dVale, widgetBind, isStaticResource) {
    const { type, value } = dVale;
    const attrVal = type === weapps_core_1.PropBindType.prop ? value : widgetBind;
    return isStaticResource
        ? `{{wxsUtils._getStaticResourceAttribute(${attrVal})}}`
        : `{{${attrVal}}}`;
}
