"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateCodeFromTpl = exports.writeLowCodeFilesForCompositeComp = exports.writeLowCodeFiles = exports.generateRouterFile = exports.pushActionToImportStringArr = exports.getActionImportStringArr = exports.getEntryImportStringArr = exports.getComponentImportStringArr = exports.getPluginImportStringArr = exports.getPluginInstancesString = exports.getListenersString = exports.getComponentSchemaString = exports.isSlot = exports.getVirtualFieldsString = exports.pullComponentToListByInstance = exports.pullActionToListByInstances = exports.getOriginPluginList = exports.getOriginComponentAndActionList = exports.generateSinglePageJsxFile = exports.generateAllPageJsxFile = exports.generateThemeVarsFile = exports.generateAppStyleLessFile = void 0;
const path_1 = __importDefault(require("path"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const lodash_1 = __importDefault(require("lodash"));
const lodash_template_1 = __importDefault(require("lodash.template"));
const react_schema_renderer_1 = require("@formily/react-schema-renderer");
const chalk_1 = __importDefault(require("chalk"));
const os_1 = __importDefault(require("os"));
const weapps_core_1 = require("../../../weapps-core");
const util_1 = require("../../util");
const config_1 = require("../../config");
const style_1 = require("../../util/style");
const common_1 = require("../../types/common");
const util_2 = require("../../util");
const dataSource_1 = require("../../../utils/dataSource");
const generateFiles_1 = require("../../util/generateFiles");
const types_1 = require("../../../types");
const util_3 = require("../../../generator/util");
function generateAppStyleLessFile(allAppDataList, appBuildDir) {
    return __awaiter(this, void 0, void 0, function* () {
        const appLessPath = path_1.default.join(appBuildDir, 'src/index.less');
        yield fs_extra_1.default.ensureFile(appLessPath);
        for (const appData of allAppDataList) {
            if (appData.codeModules) {
                let content = `@import "./lowcode/style.less";\n\n`;
                content += yield fs_extra_1.default.readFile(appLessPath, {
                    encoding: 'utf8',
                });
                yield fs_extra_1.default.writeFile(appLessPath, `${content}${os_1.default.EOL}`, {
                    encoding: 'utf8',
                });
            }
        }
    });
}
exports.generateAppStyleLessFile = generateAppStyleLessFile;
function generateThemeVarsFile(themeVars = {}, appBuildDir) {
    return __awaiter(this, void 0, void 0, function* () {
        const themeVarsPath = path_1.default.resolve(appBuildDir, './webpack/themeVars.js');
        for (const key in themeVars) {
            if (!themeVars[key]) {
                delete themeVars[key];
            }
        }
        yield fs_extra_1.default.writeFile(themeVarsPath, `module.exports = ${JSON.stringify(themeVars, null, 2)}`, {
            encoding: 'utf8',
        });
    });
}
exports.generateThemeVarsFile = generateThemeVarsFile;
function generateAllPageJsxFile(pageInstanceList, appBuildDir, rootPath, dependencies = [], extraData = {
    isComposite: false,
    compProps: {},
}, buildTypeList) {
    return __awaiter(this, void 0, void 0, function* () {
        yield Promise.all(weapps_core_1.loopDealWithFn(pageInstanceList, (pageInstance) => generateSinglePageJsxFile(pageInstance, appBuildDir, rootPath, dependencies, extraData, buildTypeList)));
    });
}
exports.generateAllPageJsxFile = generateAllPageJsxFile;
function generateSinglePageJsxFile(pageInstance, appBuildDir, rootPath, dependencies = [], extraData = {
    isComposite: false,
    compProps: {},
}, buildTypeList) {
    return __awaiter(this, void 0, void 0, function* () {
        const fixedDependencies = dependencies;
        const { componentSchemaJson, pluginInstances, listenerInstances: pageListenerInstances, style: PageStyle, data, } = pageInstance;
        const { originComponentList, originActionList } = getOriginComponentAndActionList(componentSchemaJson, fixedDependencies);
        const componentsMeta = yield util_1.getComponentsInfo(appBuildDir, dependencies);
        const originPluginList = getOriginPluginList(pluginInstances, dependencies);
        pullActionToListByInstances(pageListenerInstances, originActionList, fixedDependencies);
        const originEntryMap = {};
        originComponentList.forEach((component) => {
            if (!originEntryMap[component.materialName]) {
                const info = {
                    materialName: component.materialName,
                    version: component.materialVersion,
                    entries: component.entries,
                };
                originEntryMap[component.materialName] = info;
            }
        });
        originActionList.forEach((action) => {
            if (!originEntryMap[action.materialName]) {
                const info = {
                    materialName: action.materialName,
                    version: action.materialVersion,
                    entries: action.entries,
                };
                originEntryMap[action.materialName] = info;
            }
        });
        const entryImportStringArr = getEntryImportStringArr(Object.values(originEntryMap));
        const componentImportStringArr = getComponentImportStringArr(originComponentList);
        const actionImportStringArr = getActionImportStringArr(originActionList);
        const pluginImportStringArr = getPluginImportStringArr(originPluginList);
        const { widgets, dataBinds, componentSchema } = getComponentSchemaString(componentSchemaJson, false, componentsMeta);
        const pageStyleString = weapps_core_1.toCssText(weapps_core_1.toCssStyle(PageStyle), common_1.buildAsWebByBuildType(buildTypeList) ? 'body' : 'page');
        const templateData = {
            pageUUID: rootPath ? `${rootPath}/${pageInstance.id}` : pageInstance.id,
            pageName: pageInstance.id,
            pageStyleText: pageStyleString,
            entryImports: entryImportStringArr.join(';\n'),
            componentImports: componentImportStringArr.join(';\n'),
            pluginImports: pluginImportStringArr.join(';\n'),
            actionImports: actionImportStringArr.join(';\n'),
            pageListenerInstances: getListenersString(pageListenerInstances),
            virtualFields: getVirtualFieldsString(originComponentList),
            pluginInstances: getPluginInstancesString(pluginInstances),
            componentSchema,
            widgets,
            dataBinds,
            isComposite: extraData.isComposite,
            compProps: extraData.compProps,
            title: data.navigationBarTitleText || data.title || '',
        };
        const dest = path_1.default.resolve(appBuildDir, `./pages/${pageInstance.id}/index.jsx`);
        const template = yield fs_extra_1.default.readFile(path_1.default.resolve(config_1.appTemplateDir, './src/pages/app.tpl'), {
            encoding: 'utf8',
        });
        const jsx = lodash_template_1.default(template)(templateData);
        yield fs_extra_1.default.ensureFile(dest);
        yield fs_extra_1.default.writeFile(dest, jsx);
        const pageStyleDest = path_1.default.resolve(appBuildDir, `./pages/${pageInstance.id}/index.less`);
        const prefixStyleImport = `@import "../../lowcode/${pageInstance.id}/style.less";`;
        yield fs_extra_1.default.ensureFile(pageStyleDest);
        yield fs_extra_1.default.writeFile(pageStyleDest, prefixStyleImport + os_1.default.EOL);
    });
}
exports.generateSinglePageJsxFile = generateSinglePageJsxFile;
function getOriginComponentAndActionList(componentSchema, fixedDependencies, originComponentList = [], originActionList = []) {
    const fieldSchema = new react_schema_renderer_1.Schema(componentSchema);
    if (fieldSchema.isObject()) {
        const { 'x-props': xProps } = fieldSchema;
        if (xProps) {
            const { listenerInstances, sourceKey } = xProps;
            pullComponentToListByInstance(sourceKey, originComponentList, fixedDependencies);
            pullActionToListByInstances(listenerInstances, originActionList, fixedDependencies);
        }
        if (fieldSchema.properties) {
            for (let key in fieldSchema.properties) {
                const schema = fieldSchema.properties[key];
                const schemaJson = schema;
                getOriginComponentAndActionList(schemaJson, fixedDependencies, originComponentList, originActionList);
            }
        }
    }
    return {
        originComponentList,
        originActionList,
    };
}
exports.getOriginComponentAndActionList = getOriginComponentAndActionList;
function getOriginPluginList(pluginInstances = [], dependencies = [], originPluginList = []) {
    pluginInstances.map((instance) => {
        var _a;
        const { sourceKey } = instance;
        const { materialName, name, variableName } = util_1.getMetaInfoBySourceKey(sourceKey);
        const pluginKey = `${materialName}_${name}`;
        const isExist = originPluginList.find((item) => item.key === pluginKey);
        if (isExist) {
            return;
        }
        originPluginList.push({
            sourceKey,
            name: name || '',
            materialName: materialName || '',
            materialVersion: (_a = dependencies.find((m) => m.name === materialName)) === null || _a === void 0 ? void 0 : _a.version,
            key: pluginKey,
            variableName: variableName || '',
        });
    });
    return originPluginList;
}
exports.getOriginPluginList = getOriginPluginList;
function pullActionToListByInstances(listenerInstances, originActionList, fixedDependencies) {
    if (!listenerInstances || !listenerInstances.length) {
        return;
    }
    listenerInstances.map((pageListenerInstance) => {
        const { sourceKey, type } = pageListenerInstance;
        const { materialName, name, variableName } = util_1.getMetaInfoBySourceKey(sourceKey);
        const material = fixedDependencies.find((m) => m.name === materialName);
        const actionKey = `${materialName}_${name}`;
        const isExistAction = originActionList.find((item) => item.key === actionKey);
        if (!isExistAction) {
            originActionList.push({
                name,
                materialName,
                materialVersion: material === null || material === void 0 ? void 0 : material.version,
                key: actionKey,
                type,
                variableName,
                entries: material === null || material === void 0 ? void 0 : material.entries,
            });
        }
    });
}
exports.pullActionToListByInstances = pullActionToListByInstances;
function pullComponentToListByInstance(sourceKey, originComponentList, fixedDependencies) {
    const { materialName, name, variableName } = util_1.getMetaInfoBySourceKey(sourceKey);
    const componentKey = `${materialName}_${name}`;
    const isExistComponent = originComponentList.find((item) => item.key === componentKey);
    if (!isExistComponent) {
        const foundOne = fixedDependencies.find((m) => m.name === materialName);
        if (!foundOne)
            return;
        originComponentList.push({
            sourceKey,
            name: name || '',
            materialName: materialName || '',
            materialVersion: foundOne.version,
            key: componentKey,
            variableName: variableName || '',
            isPlainProps: !!(foundOne === null || foundOne === void 0 ? void 0 : foundOne.isPlainProps),
            entries: foundOne === null || foundOne === void 0 ? void 0 : foundOne.entries,
        });
    }
}
exports.pullComponentToListByInstance = pullComponentToListByInstance;
function getVirtualFieldsString(components) {
    const fields = components.reduce((result, component) => {
        const { name, materialName, variableName, isPlainProps } = component;
        result[`${materialName}:${name}`] = `%%%(props) => <${lodash_1.default.upperFirst(variableName)} ${isPlainProps
            ? '{...resolveComponentProps(props, 1)}'
            : '{...resolveComponentProps(props, 0)}'} pageVirtualFields={virtualFields}/>%%%`;
        return result;
    }, {});
    return JSON.stringify(fields, null, 2).replace(/"%%%(.*?)%%%"/g, function (match, expression) {
        return expression.replace(/\\"/g, '"');
    });
}
exports.getVirtualFieldsString = getVirtualFieldsString;
function isSlot(comp) {
    return comp.path && !comp['x-props'];
}
exports.isSlot = isSlot;
function getComponentSchemaString(componentSchema, isComposite = false, componentsInfoMap = {}, wrapperClass) {
    const componentInputProps = util_1.getInputProps(componentsInfoMap) || {};
    const copyJson = util_1.simpleDeepClone(componentSchema);
    const compWidgets = {};
    const compDataBinds = {};
    const componentSchemaJson = util_1.deepDealSchema(copyJson, (schema) => {
        var _a, _b, _c, _d, _e, _f, _g;
        const { 'x-props': xProps = {}, properties } = schema;
        const { dataBinds = [], commonStyle = {}, data = {}, classNameList = [], sourceKey, styleBind, classNameListBind, staticResourceAttribute = [], } = xProps;
        const componentInfo = componentsInfoMap[sourceKey];
        if ((_a = componentInfo) === null || _a === void 0 ? void 0 : _a.events) {
            schema['emitEvents'] = (_b = componentInfo) === null || _b === void 0 ? void 0 : _b.events.map((item) => item.name);
        }
        else if ((_c = componentInfo) === null || _c === void 0 ? void 0 : _c.emitEvents) {
            schema['emitEvents'] = componentInfo.emitEvents.map((item) => item.eventName);
        }
        if (!isSlot(schema) && schema.key) {
            compWidgets[schema.key] = Object.assign(Object.assign({}, data), { style: weapps_core_1.toCssStyle(commonStyle), classList: classNameList, widgetType: sourceKey, _parentId: isSlot(schema.parent)
                    ? (_e = (_d = schema === null || schema === void 0 ? void 0 : schema.parent) === null || _d === void 0 ? void 0 : _d.parent) === null || _e === void 0 ? void 0 : _e.key : (_f = schema === null || schema === void 0 ? void 0 : schema.parent) === null || _f === void 0 ? void 0 : _f.key });
            if (dataBinds.length > 0) {
                compDataBinds[schema.key] = generateDataBinds(dataBinds, isComposite);
            }
            if (styleBind) {
                if (!styleBind.bindDataPath) {
                    console.warn('无 bindDataPath', xProps);
                }
                else {
                    styleBind.propertyPath = 'style';
                    compDataBinds[schema.key] = Object.assign(Object.assign({}, (compDataBinds[schema.key] || {})), generateDataBinds([styleBind], isComposite));
                }
            }
            if (classNameListBind) {
                classNameListBind.propertyPath = 'classList';
                compDataBinds[schema.key] = Object.assign(Object.assign({}, (compDataBinds[schema.key] || {})), generateDataBinds([classNameListBind], isComposite));
            }
        }
        if (properties && weapps_core_1.isEmptyObj(properties)) {
            delete schema.properties;
        }
        delete schema.type;
        if (xProps) {
            if (isComposite) {
                if (!((_g = schema === null || schema === void 0 ? void 0 : schema.parent) === null || _g === void 0 ? void 0 : _g.parent)) {
                    if (!xProps['classNameList'])
                        xProps['classNameList'] = [];
                    xProps['classNameList'].push(wrapperClass);
                }
            }
            xProps['commonStyle'] = weapps_core_1.toCssStyle(xProps['commonStyle']);
            xProps['staticResourceAttribute'] = staticResourceAttribute;
            if (weapps_core_1.isEmptyObj(xProps['commonStyle'])) {
                delete xProps['commonStyle'];
            }
            if (weapps_core_1.isEmptyObj(xProps['style'])) {
                delete xProps['style'];
            }
            if (xProps['dataBinds'] && xProps['dataBinds'].length === 0) {
                delete xProps['dataBinds'];
            }
            if (xProps['listenerInstances'] &&
                xProps['listenerInstances'].length === 0) {
                delete xProps['listenerInstances'];
            }
            if (xProps['data']) {
                const xPropsData = xProps['data'];
                if (xPropsData._waFor && xPropsData._waFor.length === 0) {
                    delete xPropsData._waFor;
                }
                if (xPropsData.title === '') {
                    delete xPropsData.title;
                }
                if (xPropsData.staticResourceAttribute &&
                    xPropsData.staticResourceAttribute.length === 0) {
                    delete xPropsData.staticResourceAttribute;
                }
                if (weapps_core_1.isEmptyObj(xPropsData)) {
                    delete xProps['data'];
                }
            }
            if (xProps.listenerInstances) {
                xProps.listenerInstances = generateListnerInstances(xProps.listenerInstances, isComposite);
            }
            const syncProps = componentInputProps[xProps.sourceKey];
            if (syncProps) {
                if (!xProps.listenerInstances)
                    xProps.listenerInstances = [];
                Object.keys(syncProps).forEach((key) => {
                    let syncPropArr = [];
                    const syncProp = syncProps[key];
                    if (!Array.isArray(syncProp)) {
                        syncPropArr = [syncProp];
                    }
                    syncPropArr.forEach(({ changeEvent, valueFromEvent }) => {
                        xProps.listenerInstances.unshift({
                            trigger: changeEvent,
                            instanceFunction: `${config_1.REPLACE_SIGN}function({ event, forItems }) {
    const $for = forItems;
    const wid = ${isComposite ? 'this.widgets' : '$page.widgets'}.${schema.key};
    const widgetData = (forItems.forIndexes && forItems.forIndexes.length > 0) ? get(wid, forItems.forIndexes) : wid;
    widgetData.${key} = ${valueFromEvent};
  }.bind(this)${config_1.REPLACE_SIGN}`,
                        });
                    });
                });
            }
            if (xProps.dataBinds) {
                xProps.dataBinds = generateDataBinds(xProps.dataBinds, isComposite);
            }
            if (xProps.styleBind) {
                xProps.styleBind = generateDataBinds([xProps.styleBind], isComposite);
            }
            if (xProps.classNameListBind) {
                xProps.classNameListBind = generateDataBinds([xProps.classNameListBind], isComposite);
            }
        }
    });
    return {
        widgets: util_1.JsonToStringWithVariableName(compWidgets),
        dataBinds: util_1.JsonToStringWithVariableName(compDataBinds).replace(/\\r/g, '\n'),
        componentSchema: util_1.JsonToStringWithVariableName(componentSchemaJson).replace(/\\r/g, '\n'),
    };
}
exports.getComponentSchemaString = getComponentSchemaString;
function generateDataBinds(dataBinds, isComposite) {
    const dataBindFuncs = {};
    dataBinds.forEach((bind) => {
        if (!bind.bindDataPath) {
            return console.warn('无 bindDataPath', bind.propertyPath);
        }
        let funcCode = '() => {}';
        funcCode = `() => ${funcCode}`;
        if (bind.type === weapps_core_1.PropBindType.forItem) {
            funcCode = `(forItems) => forItems.${bind.bindDataPath}`;
        }
        else if (bind.type === weapps_core_1.PropBindType.scope) {
            funcCode = `() => ({__type: "scopedValue", getValue: ($scope)=>$scope.${bind.bindDataPath}})}`;
        }
        else if (bind.type === weapps_core_1.PropBindType.expression) {
            let code = bind.bindDataPath.replace(/\n/g, ' ');
            if (isComposite) {
                funcCode = `(forItems) => { const $for = forItems; return (${code.replace(/\$comp/g, 'this.$WEAPPS_COMP')})}`;
            }
            else {
                funcCode = /\$scope\./.test(code)
                    ? `(forItems, event) => ({__type: "scopedValue", getValue: ($scope) => { const $for = forItems;return (${code})}})`
                    : `(forItems, event) => { const $for = forItems;return (${code})}`;
            }
        }
        else if (bind.type === weapps_core_1.PropBindType.prop) {
            let bindDataPath = bind.bindDataPath;
            const isNegated = bindDataPath.startsWith('!');
            if (isNegated)
                bindDataPath = bindDataPath.replace(/^!/, '');
            if (isComposite) {
                funcCode = `() => ${isNegated ? '!' : ''}this.$WEAPPS_COMP.props.data.${bindDataPath}`;
            }
            else {
                funcCode = `() => ${isNegated ? '!' : ''}$page.props.data.${bindDataPath}`;
            }
        }
        else {
            const PREFIX_MAP = {
                [weapps_core_1.PropBindType.state]: 'state',
                [weapps_core_1.PropBindType.computed]: 'computed',
                [weapps_core_1.PropBindType.dataVar]: 'dataVar',
                [weapps_core_1.PropBindType.stateData]: 'dataset.state',
                [weapps_core_1.PropBindType.paramData]: 'dataset.params',
            };
            switch (bind.type) {
                case weapps_core_1.PropBindType.state:
                case weapps_core_1.PropBindType.computed:
                case weapps_core_1.PropBindType.dataVar:
                case weapps_core_1.PropBindType.stateData:
                case weapps_core_1.PropBindType.paramData: {
                    if (bind.bindDataPath.startsWith('global.')) {
                        funcCode = bind.bindDataPath.replace(/^global./, `app.${PREFIX_MAP[bind.type]}.`);
                    }
                    else {
                        if (isComposite) {
                            funcCode = bind.bindDataPath
                                .replace(/^comp-\w+./, `this.$WEAPPS_COMP.${PREFIX_MAP[bind.type]}.`)
                                .replace(/^\$comp_\w+./, `this.$WEAPPS_COMP.${PREFIX_MAP[bind.type]}.`);
                        }
                        else {
                            funcCode = bind.bindDataPath.replace(/^\w+./, `$page.${PREFIX_MAP[bind.type]}.`);
                        }
                    }
                    funcCode = `() => ${funcCode}`;
                    break;
                }
            }
        }
        dataBindFuncs[bind.propertyPath] = `${config_1.REPLACE_SIGN}${funcCode}${config_1.REPLACE_SIGN}`;
    });
    return dataBindFuncs;
}
function generateListnerInstances(listenerInstances, isComposite = false) {
    return listenerInstances.map((listener) => {
        var _a, _b, _c, _d, _e;
        const generatedListener = {
            key: listener.key,
            trigger: listener.trigger,
            isCapturePhase: listener.isCapturePhase,
            noPropagation: listener.noPropagation,
        };
        if (listener.type === weapps_core_1.ActionType.Material) {
            const { sourceKey } = listener;
            const { variableName } = util_1.getMetaInfoBySourceKey(sourceKey);
            generatedListener.instanceFunction = `${config_1.REPLACE_SIGN}${variableName}${config_1.REPLACE_SIGN}`;
        }
        else if (listener.type === weapps_core_1.ActionType.Platform) {
            generatedListener.instanceFunction = `${config_1.REPLACE_SIGN}function({data}) { return app.${((_a = listener.handler) === null || _a === void 0 ? void 0 : _a.name) || listener.jsCode}(data) }${config_1.REPLACE_SIGN}`;
        }
        else if (listener.type === weapps_core_1.ActionType.DataSource) {
            generatedListener.instanceFunction = `${config_1.REPLACE_SIGN}function({data}) { return app.cloud.callDataSource(data) }${config_1.REPLACE_SIGN}`;
        }
        else if (listener.type === weapps_core_1.ActionType.PropEvent) {
            if (isComposite) {
                generatedListener.instanceFunction = `${config_1.REPLACE_SIGN}function({data}) { this.props.emit('${((_b = listener.handler) === null || _b === void 0 ? void 0 : _b.name) || listener.jsCode}', data.target) }.bind(this)${config_1.REPLACE_SIGN}`;
            }
            else {
                generatedListener.instanceFunction = `${config_1.REPLACE_SIGN}function(...args) { $page.props.events.${((_c = listener.handler) === null || _c === void 0 ? void 0 : _c.name) || listener.jsCode}.apply(null, args) }${config_1.REPLACE_SIGN}`;
            }
        }
        else {
            if (isComposite) {
                generatedListener.instanceFunction = `${config_1.REPLACE_SIGN}this.handler.${((_d = listener.handler) === null || _d === void 0 ? void 0 : _d.name) || listener.jsCode}.bind(this)${config_1.REPLACE_SIGN}`;
            }
            else {
                generatedListener.instanceFunction = `${config_1.REPLACE_SIGN}handler.${((_e = listener.handler) === null || _e === void 0 ? void 0 : _e.name) || listener.jsCode}${config_1.REPLACE_SIGN}`;
            }
        }
        if (!weapps_core_1.isEmptyObj(listener.data)) {
            generatedListener.data = listener.data;
        }
        if (listener.dataBinds && listener.dataBinds.length > 0) {
            generatedListener.dataBinds = generateDataBinds(listener.dataBinds, isComposite);
        }
        return generatedListener;
    });
}
function getListenersString(listeners = [], isComposite = false) {
    return util_1.JsonToStringWithVariableName(generateListnerInstances(listeners, isComposite));
}
exports.getListenersString = getListenersString;
function getPluginInstancesString(instances) {
    if (!instances || !instances.length) {
        return '[]';
    }
    const copyInstances = util_1.simpleDeepClone(instances);
    copyInstances.map((itemInstance) => {
        const { sourceKey } = itemInstance;
        const { variableName } = util_1.getMetaInfoBySourceKey(sourceKey);
        itemInstance.instanceFunction = `${config_1.REPLACE_SIGN}${variableName}${config_1.REPLACE_SIGN}`;
    });
    return util_1.JsonToStringWithVariableName(copyInstances);
}
exports.getPluginInstancesString = getPluginInstancesString;
function getPluginImportStringArr(plugins, pluginImportStringArr = []) {
    plugins.map((plugin) => __awaiter(this, void 0, void 0, function* () {
        const { name, materialName, materialVersion, variableName } = plugin;
        const importString = `import ${variableName} from 'libraries/${`${materialName}@${materialVersion}`}/plugins/${name}'`;
        if (pluginImportStringArr.includes(importString)) {
            return;
        }
        pluginImportStringArr.push(importString);
    }));
    return pluginImportStringArr;
}
exports.getPluginImportStringArr = getPluginImportStringArr;
function getComponentImportStringArr(components, componentImportStringArr = []) {
    components.map((component) => __awaiter(this, void 0, void 0, function* () {
        const { name, materialName, materialVersion, variableName, entries } = component;
        let importString = '';
        if (entries === null || entries === void 0 ? void 0 : entries.components) {
            importString = `import { ${name} as ${lodash_1.default.upperFirst(variableName)} } from '${path_1.default.posix.join(`libraries/${materialName}@${materialVersion}`, entries === null || entries === void 0 ? void 0 : entries.components)}'`;
        }
        else if (entries === null || entries === void 0 ? void 0 : entries.entry) {
            const componentsLibVariableName = util_3.camelcase(`${materialName}`);
            importString = `const { ${name}: ${lodash_1.default.upperFirst(variableName)} } = ${componentsLibVariableName}.components`;
        }
        else {
            importString = `import ${lodash_1.default.upperFirst(variableName)} from 'libraries/${`${materialName}@${materialVersion}`}/components/${name}'`;
        }
        if (!componentImportStringArr.includes(importString)) {
            componentImportStringArr.push(importString);
        }
    }));
    return componentImportStringArr.filter((item) => !!item);
}
exports.getComponentImportStringArr = getComponentImportStringArr;
function getEntryImportStringArr(materialInfoList = [], entryImportStringArr = []) {
    materialInfoList.forEach(({ materialName, version, entries }) => {
        if (!((entries === null || entries === void 0 ? void 0 : entries.actions) && (entries === null || entries === void 0 ? void 0 : entries.components)) && (entries === null || entries === void 0 ? void 0 : entries.entry)) {
            const componentsLibVariableName = util_3.camelcase(`${materialName}`);
            const importComponentLibString = `import ${componentsLibVariableName} from '${path_1.default.posix.join(`libraries/${materialName}@${version}`, entries === null || entries === void 0 ? void 0 : entries.entry)}'`;
            if (!entryImportStringArr.includes(importComponentLibString)) {
                entryImportStringArr.push(importComponentLibString);
            }
        }
    });
    return entryImportStringArr || [];
}
exports.getEntryImportStringArr = getEntryImportStringArr;
function getActionImportStringArr(originActionList, actionImportStringArr = []) {
    originActionList.map((action) => {
        if (action.type === weapps_core_1.ActionType.Material) {
            pushActionToImportStringArr(action, actionImportStringArr);
        }
    });
    return actionImportStringArr;
}
exports.getActionImportStringArr = getActionImportStringArr;
function pushActionToImportStringArr(listenerInstance, actionImportStringArr) {
    const { name, materialName, materialVersion, variableName, entries } = listenerInstance;
    const componentsLibVariableName = util_3.camelcase(`${materialName}`);
    const importString = (entries === null || entries === void 0 ? void 0 : entries.actions) ? `import { ${name} as ${variableName} } from '${path_1.default.posix.join(`libraries/${materialName}@${materialVersion}`, entries === null || entries === void 0 ? void 0 : entries.actions)}'`
        : (entries === null || entries === void 0 ? void 0 : entries.entry) ? `const { ${name}: ${variableName} } = ${componentsLibVariableName}.actions`
            : `import ${variableName} from 'libraries/${materialName}@${materialVersion}/actions/${name}'`;
    if (actionImportStringArr.includes(importString)) {
        return;
    }
    actionImportStringArr.push(importString);
}
exports.pushActionToImportStringArr = pushActionToImportStringArr;
function generateRouterFile(allAppDataList, appBuildDir, basename = '', buildTypeList = []) {
    return __awaiter(this, void 0, void 0, function* () {
        const routerImports = [];
        const routerRenders = [];
        const mountApis = [];
        yield Promise.all(allAppDataList.map((data) => __awaiter(this, void 0, void 0, function* () {
            const { pageInstanceList, rootPath = '' } = data;
            const pageFilePath = rootPath ? `packages/${rootPath}/` : '';
            mountApis.push(`import '${pageFilePath}app/mountAppApis';`);
            weapps_core_1.loopDealWithFn(pageInstanceList, (pageInstance) => {
                const pageId = [rootPath, pageInstance.id].filter((i) => i).join('_');
                if (pageInstance.isHome && !rootPath) {
                    routerRenders.push(`<Redirect from="/" exact to="/${pageId}"/>`);
                }
                routerImports.push(`import Page${pageId} from '${pageFilePath}pages/${pageInstance.id}/index';`);
                routerRenders.push(`<Route path="/${pageId}" component={Page${pageId}}/>`);
            });
        })));
        const routerTemplate = yield fs_extra_1.default.readFile(path_1.default.resolve(config_1.appTemplateDir, './src/router/index.tpl'), {
            encoding: 'utf8',
        });
        const routerIndexStr = lodash_template_1.default(routerTemplate)({
            routerImports: routerImports.join('\n'),
            routerRenders: routerRenders.join('\n'),
            mountApis: mountApis.join('\n'),
            basename: basename,
        });
        const dest = path_1.default.resolve(appBuildDir, `src/router/index.jsx`);
        yield fs_extra_1.default.ensureFile(dest);
        yield fs_extra_1.default.writeFile(dest, routerIndexStr);
    });
}
exports.generateRouterFile = generateRouterFile;
function writeLowCodeFiles(appData, appBuildDir) {
    return __awaiter(this, void 0, void 0, function* () {
        const lowcodeRootDir = path_1.default.join(appBuildDir, 'lowcode');
        console.log(chalk_1.default.blue.bold('Writing lowcode files:'));
        generateFiles_1.cleanDir(lowcodeRootDir, []);
        style_1.generateDefaultTheme(appData);
        const themeCode = appData.codeModules.find((mod) => mod.type === 'theme');
        yield Promise.all(appData.codeModules.map((m) => writeCode2file(m, 'global')));
        yield Promise.all(weapps_core_1.loopDealWithFn(appData.pageInstanceList, (page) => __awaiter(this, void 0, void 0, function* () {
            style_1.generateDefaultStyle(page);
            yield page.codeModules
                .filter((m) => m.name !== '____index____')
                .forEach((m) => writeCode2file(m, page.id));
        })));
        function writeCode2file(mod, pageId) {
            return __awaiter(this, void 0, void 0, function* () {
                const file = path_1.default.join(lowcodeRootDir, weapps_core_1.getCodeModuleFilePath(pageId, mod));
                let weappsApiPrefix = '';
                if (mod.type !== 'theme') {
                    weappsApiPrefix =
                        mod.type !== 'style'
                            ? `import { app, $page } from '${path_1.default
                                .relative(path_1.default.dirname(file), appBuildDir + '/app/global-api')
                                .replace(/\\/g, '/')}';`
                            : ``;
                }
                console.log(chalk_1.default.green(file));
                let code = mod.code;
                if (mod.type === 'style') {
                    code = yield style_1.processLess(((themeCode === null || themeCode === void 0 ? void 0 : themeCode.code) || style_1.defaultThemeCode) + code);
                }
                yield fs_extra_1.default.ensureFile(file);
                yield fs_extra_1.default.writeFile(file, weappsApiPrefix + os_1.default.EOL + code);
            });
        }
    });
}
exports.writeLowCodeFiles = writeLowCodeFiles;
function writeLowCodeFilesForCompositeComp(compositeGroups, appBuildDir) {
    return __awaiter(this, void 0, void 0, function* () {
        const lowcodeRootDir = path_1.default.join(appBuildDir, 'src', 'lowcode', 'composite');
        console.log(chalk_1.default.blue.bold('Writing composite component lowcode files:'));
        yield Promise.all(compositeGroups.map((gItem) => __awaiter(this, void 0, void 0, function* () {
            yield util_2.writeLibCommonRes2file(gItem, path_1.default.join(appBuildDir, 'src', 'libraries', `${gItem.name}@${gItem.version}`, 'libCommonRes'));
            const compLibCommonResource = gItem.compLibCommonResource;
            let themeCode = '';
            if (compLibCommonResource) {
                themeCode = `
          ${compLibCommonResource.theme.variable || ''}
          ${compLibCommonResource.class || ''}
          ${compLibCommonResource.theme.class || ''}
        `;
            }
            return yield Promise.all(gItem.components.map((component) => __awaiter(this, void 0, void 0, function* () {
                let cItem = component;
                return yield cItem.lowCodes.forEach((m) => writeCode2file(m, path_1.default.join(appBuildDir, 'src', 'libraries', `${gItem.name}@${gItem.version}`, 'components', cItem.name, 'lowcode'), cItem, themeCode));
            })));
        })));
        function writeCode2file(mod, lowcodeDir, comp, themeCode) {
            return __awaiter(this, void 0, void 0, function* () {
                const pageId = comp.name + '_' + comp.id;
                const file = path_1.default.join(lowcodeDir, weapps_core_1.getCodeModuleFilePath(pageId, mod));
                yield fs_extra_1.default.ensureFile(file);
                let codeContent = '';
                if (mod.type === 'style') {
                    codeContent = `.${weapps_core_1.getCompositedComponentClass(comp)} { \n${themeCode ? themeCode : ''}\n${mod.code}\n }`;
                    try {
                        codeContent = yield style_1.processLess(`${codeContent}`);
                    }
                    catch (e) {
                        console.error(`样式转换失败 [${pageId}] :`, e, codeContent);
                    }
                }
                else {
                    codeContent = `import { app } from 'app/global-api'
      ${mod.code.replace(/\$comp/g, 'this.$WEAPPS_COMP')};`;
                }
                yield fs_extra_1.default.writeFile(file, codeContent);
            });
        }
    });
}
exports.writeLowCodeFilesForCompositeComp = writeLowCodeFilesForCompositeComp;
function generateCodeFromTpl(appData, appBuildDir, dependencies, appKey, rootPath, deployMode, buildTypeList, extraData) {
    return __awaiter(this, void 0, void 0, function* () {
        const pageIds = [];
        const pageModules = {};
        weapps_core_1.loopDealWithFn(appData.pageInstanceList, (p) => {
            pageIds.push(p.id);
            pageModules[p.id] = p.codeModules;
        });
        const yyptConfig = yield util_2.getYyptConfigInfo(extraData);
        const templatesData = {
            'store/index.js': {
                pageIds,
                appId: appKey,
                rootPath: rootPath,
            },
            'store/computed.js': {
                pageIds,
            },
            'app/handlers.js': {
                pageModules,
            },
            'app/material-actions.js': {
                materials: dependencies || [],
                _: lodash_1.default,
            },
            'app/common.js': {
                mods: appData.codeModules
                    .filter((m) => m.type === 'normal-module' && m.name !== '____index____')
                    .map((m) => m.name),
            },
            'datasources/config.js.tpl': {
                appID: appKey,
                envID: appData.envId,
                isProd: deployMode === types_1.DEPLOY_MODE.UPLOAD,
            },
            'datasources/datasource-profiles.js.tpl': {
                datasourceProfiles: util_1.JsonToStringWithVariableName(dataSource_1.getDatasourceProfiles(appData.datasources || [])),
            },
            'datasources/dataset-profiles.js.tpl': {
                datasetProfiles: util_1.JsonToStringWithVariableName(dataSource_1.getDatasetProfiles(appData, [appData])),
            },
            'handlers/utils/common.js': {
                isAdminPortal: common_1.buildAsAdminPortalByBuildType(buildTypeList),
            },
        };
        if (!rootPath) {
            templatesData['index.jsx'] = Object.assign(Object.assign({}, yyptConfig), { adminPortalKey: deployMode === types_1.DEPLOY_MODE.PREVIEW ? `${appKey}-preview` : appKey });
        }
        console.log(chalk_1.default.blue.bold('Generating code by templates:'));
        for (const file in templatesData) {
            const tplStr = yield fs_extra_1.default.readFile(path_1.default.join(config_1.appTemplateDir, 'src', file), {
                encoding: 'utf8',
            });
            const generatedCode = lodash_template_1.default(tplStr, {
                interpolate: /<%=([\s\S]+?)%>/g,
            })(templatesData[file]);
            const outFile = path_1.default.resolve(appBuildDir, file.replace(/.tpl$/, ''));
            const outTplPath = /.tpl$/.test(file)
                ? path_1.default.resolve(appBuildDir, file)
                : null;
            yield fs_extra_1.default.ensureFile(outFile);
            console.log(outFile);
            yield fs_extra_1.default.writeFile(outFile, generatedCode);
            if (outTplPath) {
                fs_extra_1.default.removeSync(outTplPath);
            }
        }
    });
}
exports.generateCodeFromTpl = generateCodeFromTpl;
