"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.fixProcessCwd = exports.getStoreModuleList = exports.createNodeModulesSoftLink = exports.getCompileDirs = void 0;
const path_1 = __importDefault(require("path"));
const os_1 = __importDefault(require("os"));
const symlink_dir_1 = __importDefault(require("symlink-dir"));
const util_1 = require("../../util");
const fs_extra_1 = __importDefault(require("fs-extra"));
const weapps_core_1 = require("../../../weapps-core");
function getCompileDirs(appKey = 'test') {
    const materialsDir = path_1.default.join(os_1.default.homedir(), '.weapps-materials');
    const rootBuildDir = path_1.default.join(os_1.default.homedir(), '.weapps-build');
    const appBuildDir = path_1.default.join(rootBuildDir, appKey);
    return {
        rootBuildDir,
        appBuildDir,
        materialsDir,
    };
}
exports.getCompileDirs = getCompileDirs;
function createNodeModulesSoftLink(appBuildDir, nodeModulesPath) {
    return __awaiter(this, void 0, void 0, function* () {
        const destPath = path_1.default.join(appBuildDir, 'node_modules');
        if (!fs_extra_1.default.existsSync(destPath)) {
            util_1.promiseWrapper(symlink_dir_1.default(nodeModulesPath, destPath));
        }
    });
}
exports.createNodeModulesSoftLink = createNodeModulesSoftLink;
function getStoreModuleList(appData) {
    const storeModuleList = [];
    extractRematchModules(appData.codeModules);
    weapps_core_1.loopDealWithFn(appData.pageInstanceList, (pageInstance) => {
        extractRematchModules(pageInstance.codeModules);
    });
    return storeModuleList;
    function extractRematchModules(mods) {
        mods
            .filter(m => m.type === 'rematch')
            .forEach(m => {
            storeModuleList.push({
                namespace: m.name,
                code: m.code,
            });
        });
    }
}
exports.getStoreModuleList = getStoreModuleList;
function fixProcessCwd(appBuilderDir) {
    const currentCwd = process.cwd();
    const nodeModulesWaPath = path_1.default.resolve(currentCwd, 'node_modules/.bin/wa');
    if (fs_extra_1.default.pathExistsSync(nodeModulesWaPath)) {
        return;
    }
    appBuilderDir && process.chdir(appBuilderDir);
}
exports.fixProcessCwd = fixProcessCwd;
